/***************************************************************************
                          molecule.cpp  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Jerome PANSANEL
    email                : pansanel@pansanel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "molecule.h"
#include <iostream.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#define SIZE_SYMBOL 4
#define SIZE_MULTIPLE 16
#define TABLE_LENGTH 16

molecule::molecule(void)
{
	molecular_weight = 0;
}

molecule::~molecule(void)
{
}

/*Add a new element in the table, after checking that the element isn't already in. this part initialize the atomic symbol and their multiplicity*/
void molecule::add_element(char * elem, float scale)
{
  short int i = 0;
  while (*compound[i].get_symbol() != '\0')
	{
if (!strcmp(compound[i].get_symbol(), elem))
	{
  break;
		}
     else
		{
		  i++;
		}
	}
  compound[i].set_symbol(elem);
  compound[i].set_multiple(compound[i].get_multiple()+scale);
}

/*Parse the molecule and cut it in element, with the appropriate multiplicity*/
void molecule::parse(char * formula)
{
  short int i = 0;
  while(formula[i] != '\0')
    {
      if(isupper(formula[i]))
        {
	  char * temp = new char[SIZE_SYMBOL];
	  *temp = '\0';
          strncat(temp, formula+i,1);
          while(islower(*(formula+i+1)))
            {
              i++;
              strncat(temp, formula+i, 1);
            }
          i++;
	  float n = 0;
          if(isdigit(*(formula+i)))
            {
              char * numeric = new char[SIZE_MULTIPLE];
	      *numeric = '\0';
              while(isdigit(*(formula+i)) || (*(formula+i) == '.'))
                {
                strncat(numeric, formula+i, 1);
                i++;
		}
              n = atof(numeric);
	      numeric = NULL;
	      delete[] numeric;
            }
          else
            {
	      n = 1;
            }
	  add_element(temp, n);
	  temp = NULL;
	  delete[] temp;
	 }
      else
	{
	  cerr << "Unknown character: " << formula[i] << endl;
	  exit(8);
	}
    }
}

/*Initialize the atomic weight of each element in the molecule.*/
void molecule::set_atom_mass(char * file)
{
  fstream table;
  table.open(file, ios::in);
  if (!table.is_open())
    {
      cerr << "mwcalc: " << file << ": No such file or directory" << endl;
      exit(0);
    }
  char * table_ptr = new char[TABLE_LENGTH];
  *table_ptr = '\0';
  short int i = 0;
  while(*compound[i].get_symbol() != '\0')
    {
      table.seekg(0);
      while(1)
	{
	  table.getline(table_ptr, TABLE_LENGTH, '\n');
	  if (!strcmp(compound[i].get_symbol(), table_ptr))
	    {
	      table.getline(table_ptr, TABLE_LENGTH, '\n');
	      compound[i].set_mass(atof(table_ptr));
	      break;
	    }
	  else
	    {
	      table.getline(table_ptr, TABLE_LENGTH, '\n');
	    }
	  if (table.eof())
	    {
	      cerr << "Unknown element: " << compound[i].get_symbol() << endl;
	      exit(0);
	    }
	}
      i++;
    }
    table_ptr = NULL;
    delete[] table_ptr;
    table.close();
 }

/*Calculate the molecular weight of the molecule.*/
void molecule::calcul_weight(void)
{
	for (int i = 0; *compound[i].get_symbol() != '\0'; i++)
	{
		molecular_weight += compound[i].get_mass() * compound[i].get_multiple();
	}
}

float molecule::get_weight(void)
{
	return(molecular_weight);
}

void molecule::get_parsed_formula(void)
{
	short int i = 0;
	while(*compound[i].get_symbol() != '\0')
	{
		cout << compound[i].get_symbol();
		cout << compound[i].get_multiple();
		i++;
	}
}

void molecule::get_elementar_analysis(void)
{
short int i = 0;
while(*compound[i].get_symbol() != '\0')
  {
  cout << compound[i].get_symbol() << ": " << (compound[i].get_mass() * compound[i].get_multiple() * 100 / molecular_weight) << " % "; 
  i++;
  }
}
